@@ -1,3 +1,6 @@
+from ast import Dict
+import os
+import pathlib
 from scrapy.crawler import Crawler
 from scrapy.statscollectors import StatsCollector
 from typing import Self
@@ -7,13 +10,17 @@ from scrapy.exceptions import NotConfigured
 from scrapy.http import Request, Response
 from scrapy.core.downloader import Slot
 from twisted.internet import task
+from scrapy.logformatter import LogFormatter
+
+import scrapy_zen.signals
+from scrapy_zen.logging import ZenBufferedLogHandler
 
 
 logger = logging.getLogger(__name__)
 
 
 
-class ZenExtension:
+class ZenStats:
     """
     Allows to calculate average latency across requests and shows logstats
     """
@@ -87,6 +94,35 @@ class ZenExtension:
         return (pages / mins_elapsed), (items / mins_elapsed)
 
 
+
+class ZenBufferedLogs:
+
+    def __init__(self, handler: ZenBufferedLogHandler):
+        logging.getLogger().addHandler(handler)
+        self.handler = handler
+
+    @classmethod
+    def from_crawler(cls, crawler: Crawler) -> Self:
+        if not crawler.settings.getbool("ZEN_BUFFEREDLOGS_ENABLED"):
+            raise NotConfigured
+        log_file = crawler.settings.get("LOG_FILE")
+        if not log_file:
+            raise NotConfigured
+
+        buffer_length = crawler.settings.getint("ZEN_BUFFEREDLOGS_LENGTH", 30)
+        filename = pathlib.Path(log_file).stem + "_buffered.log"
+        handler = ZenBufferedLogHandler(filename, buffer_length)
+        handler.setFormatter(crawler.logformatter)
+
+        ext = cls(handler)
+
+        crawler.signals.connect(ext.new_item, signal=scrapy_zen.signals.new_item)
+        return ext
+
+    def new_item(self, item: Dict, spider: Spider):
+        self.handler.flush()
+
+
 class ZenAutoThrottle:
     def __init__(self, crawler: Crawler):
         self.crawler: Crawler = crawler
